/* $VER: DM_PaternRename.rexx 1.3 (13.6.04) by J. Tierney

  DiskMaster II Pattern Rename  v1.3
  J. Tierney <jodytierney@earthlink.net>
  2/25/03, 6/13/04

  Purpose:  Rename selected file(s) according to a pattern.

  Patterns:  *  = Rest of original filename.
             ?  = One character of the original filename, unmodified.
             X  = Skip one character of the original filename.
             +  = One character of the original filename, uppercase.
             -  = One character of the original filename, lowercase.

             ^<text>^ = Insert <text> into the new filename.
             /<text>/ = Original filename from current position up to <text>.
                        The current position will be at the first character of
                        <text>.
             \<text>\ = Skip up to the beginning of <text>.
             #<num>#  = Add an index number to the new string.
                        NOTE:  Only one index in a template is allowed.
                        Ex:  #004#

  Usage:  Rexx DM_PatternRename.rexx <default pattern>

  Examples:
    If the original file name is "pic1.JPG", then:

    Pattern: +*
    Result:  Pic1.JPG

    Pattern: +????---
    Result:  Pic1.jpg

    Pattern: ???^00^?*
    Result:  pic001.JPG

    Pattern: XXX^Glacier00^??---
    Result:  Glacier001.jpg

*/

OPTIONS RESULTS

'STATUS S'
IF result = 0 THEN EXIT

PARSE ARG defpat
IF defpat = '' THEN defpat = '*'
IF LEFT(defpat, 1) ~= '"' THEN defpat = '"' || defpat || '"'

bdytxt = '"Please specify pattern:|'
bdytxt = bdytxt || '* = Rest of orig. name|'
bdytxt = bdytxt || '? = One char. of orig. name, unchanged|'
bdytxt = bdytxt || '+ = One char. of orig. name, uppercase|'
bdytxt = bdytxt || '- = One char. of orig. name, lowercase|'
bdytxt = bdytxt || 'X = Skip one char.|'
bdytxt = bdytxt || '^<text>^ = Insert <text>|'
bdytxt = bdytxt || '/<text>/ = Orig. name from current to <text>|'
bdytxt = bdytxt || '\<text>\ = Skip to <text> in orig. name|'
bdytxt = bdytxt || '#<num># = Index num. template  Ex: #004#"'

'CHOOSE TITLE="Pattern Rename" TEXT=' || bdytxt 'BUTTONS="Okay|Cancel" STRING=' || defpat
IF rc = 1 THEN EXIT 0
pat = result
IF pat = '' | pat ='*' THEN EXIT 0
pat = ParseTemplate(pat)

STATUS P
path = result
IF RIGHT(path, 1) ~= ':' THEN path = path || '/'

DIRLIST VAR dlist SEL

DO i = 1 TO dlist.name.0
  oldname = dlist.name.i
  newname = AlterStr(oldname, pat)
/*  address command 'echo >>t:out' '"' || path || oldname 'TO' path || newname '"' */
  ADDRESS COMMAND 'C:Rename "' || path || oldname || '" TO "' || path || newname || '"'
END

'DESELECT *'
EXIT 0


/* --- Procedures --- */

ParseTemplate: PROCEDURE EXPOSE index.
  PARSE ARG tmpl

  /* --- Catch/Initialize Indexing --- */
  newtmpl = tmpl
  x = POS('#', tmpl)
  IF x ~= 0 THEN DO
    IF (Within('^', x, tmpl) | Within('/', x, tmpl) | Within('\', x, tmpl)) = 0 THEN DO
      /* The # isn't inside a ^^, //, or \\ block.  Go play. */
      y = POS('#', tmpl, x + 1)
      IF y = 0 THEN y = LENGTH(tmpl) + 1
      index.len = y - x - 1
      index.num = SUBSTR(tmpl, x + 1, index.len)
      newtmpl = DELSTR(tmpl, x + 1, index.len + 1)
    END
  END
RETURN newtmpl

AlterStr:  PROCEDURE EXPOSE index.
  PARSE ARG str, tmpl

  newstr = ''
  sp = 1
  tp = 1
  l = LENGTH(tmpl)

  DO i = 1 TO l
    cmd = UPPER(SUBSTR(tmpl, i, 1))
    SELECT
      WHEN cmd = '*' THEN DO
        /* --- Rest of string. --- */
        newstr = newstr || SUBSTR(str, sp)
        sp = LENGTH(str)
      END

      WHEN cmd = '?' THEN DO
        /* --- One character, unmodified. --- */
        newstr = newstr || SUBSTR(str, sp, 1)
        sp = sp + 1
      END

      WHEN cmd = 'X' THEN DO
        /* --- Ignore character. --- */
        sp = sp + 1
      END

      WHEN cmd = '+' THEN DO
        /* --- Uppercase character. --- */
        newstr = newstr || UPPER(SUBSTR(str, sp, 1))
        sp = sp + 1
      END

      WHEN cmd = '-' THEN DO
        /* --- Lowercase character. --- */
        newstr = newstr || Lower(SUBSTR(str, sp, 1))
        sp = sp + 1
      END

      WHEN cmd = '^' THEN DO
        /* --- Insert text. --- */
        i = i + 1
        x = POS('^', tmpl, i)
        IF x = 0 THEN x = LENGTH(tmpl) + 1
        newstr = newstr || SUBSTR(tmpl, i, x - i)
        i = x
      END

      WHEN cmd = '/' THEN DO
        /* --- Copy up to (but not including) this text. --- */
        i = i + 1
        x = POS('/', tmpl, i)
        IF x = 0 THEN x = LENGTH(tmpl) + 1
        fnd = UPPER(SUBSTR(tmpl, i, x - i))
        i = x

        x = POS(fnd, UPPER(str), sp)
        IF x ~= 0 THEN DO
          newstr = newstr || SUBSTR(str, sp, x - sp)
          sp = x
        END
      END

      WHEN cmd = '\' THEN DO
        /* --- Skip up to the beginning of this text. --- */
        i = i + 1
        x = POS('\', tmpl, i)
        IF x = 0 THEN x = LENGTH(tmpl) + 1
        fnd = UPPER(SUBSTR(tmpl, i, x - i))
        i = x

        x = POS(fnd, UPPER(str), sp)
        IF x ~= 0 THEN sp = x
      END

      WHEN cmd = '#' THEN DO
        newstr = newstr || RIGHT(index.num, index.len, '0')
        index.num = index.num + 1
      END

      OTHERWISE NOP
    END
  END
RETURN newstr

Lower:  PROCEDURE
  PARSE ARG str

  newstr = TRANSLATE(str, XRANGE(, '40'x) || XRANGE('61'x, '7a'x) || XRANGE('5B'x,), XRANGE())
RETURN newstr

Within:  PROCEDURE
  ARG chars, pos, str
  inside = 0
  sp = 1
  x = 1

  DO UNTIL x = 0
    x = POS(chars, str, sp)
    sp = x + 1
    IF x ~= 0 THEN DO
      IF x < pos THEN DO
        x = POS(chars, str, sp)
        sp = x + 1
        IF x > pos THEN DO
          inside = 1
          LEAVE
        END
      END
      ELSE LEAVE
    END
  END
RETURN inside

